// cpukeeper
// Interface to the Opencores.org Mini-RISC CPU core module
// For more information, visit www.opencores.org.

module cpukeeper(clk, reset,
		 positive_vblank,
		 instr_addr, instr_data,
		 maccess_addr, maccess_read, maccess_we, maccess_write,
		 kbd_data, status_read, cfg_write, cfg_we,
		 cpu_timer);

   input clk, reset;
   input positive_vblank;

   output [10:0] instr_addr;
   input [11:0] instr_data;

   output [7:0] maccess_addr;
   input [5:0] 	maccess_read;
   output [5:0] maccess_we, maccess_write;
   input [7:0] 	kbd_data;
   input [7:0] 	status_read;
   output [7:0] cfg_write;
   output 	cfg_we;
   output [7:0] cpu_timer;

   wire [7:0] portb_read = { 2'b00, maccess_read };
   wire [7:0] portb_write;
   wire [7:0] trisa, trisb, trisc;

   wire irq_en;
   wire minirisc_reset;
   assign minirisc_reset = reset || (irq_en && positive_vblank);

   mrisc mycpu(clk, minirisc_reset,
	       instr_addr, instr_data,
	       kbd_data, portb_read, status_read,
	       maccess_addr, portb_write, cfg_write,
	       trisa, trisb, trisc,
	       1'b0, 1'b0, cpu_timer);

   assign maccess_write = portb_write[5:0];
   assign maccess_we = ~trisb[5:0];
   assign cfg_we = ~trisc[0];
   assign irq_en = trisc[1];

endmodule

`include "risc_core.v"
`include "register_file_x.v"
`include "primitives.v"
`include "presclr_wdt.v"
`include "alu.v"
