/////////////////////////////////////////////////////////////////////
////                                                             ////
////  Mini-RISC-1                                                ////
////  Register File                                              ////
////                                                             ////
////  Author: Rudolf Usselmann                                   ////
////          russelmann@hotmail.com                             ////
////                                                             ////
/////////////////////////////////////////////////////////////////////
////                                                             ////
//// Copyright (C) 2000 Rudolf Usselmann                         ////
////                    russelmann@hotmail.com                   ////
////                                                             ////
//// This source file may be used and distributed without        ////
//// restriction provided that this copyright statement is not   ////
//// removed from the file and that any derivative work contains ////
//// the original copyright notice and the associated disclaimer.////
////                                                             ////
//// THIS SOURCE FILE IS PROVIDED "AS IS" AND WITHOUT ANY        ////
//// EXPRESS OR IMPLIED WARRANTIES, INCLUDING, WITHOUT           ////
//// LIMITATION, THE IMPLIED WARRANTIES OF MERCHANTIBILITY AND   ////
//// FITNESS FOR A PARTICULAR PURPOSE.                           ////
////                                                             ////
/////////////////////////////////////////////////////////////////////


`timescale 1ns / 10ps

module register_file(	clk,
			rf_rd_bnk, rf_rd_addr, rf_rd_data,
			rf_we, rf_wr_bnk, rf_wr_addr, rf_wr_data);

input		clk;
input  [1:0]	rf_rd_bnk;
input  [4:0]	rf_rd_addr;
output [7:0]	rf_rd_data;
input		rf_we;
input  [1:0]	rf_wr_bnk;
input  [4:0]	rf_wr_addr;
input  [7:0]	rf_wr_data;

wire		clk;
wire [7:0]	rf_rd_data;
wire [6:0]	rd_addr;
wire [6:0]	wr_addr;
wire [7:0]	rf_rd_data_mem;
wire		rd_wr_addr_equal;

// Memories
reg [7:0] lowmem [15:0];
reg [7:0] highmem [63:0];

reg [7:0] rd_latch;
assign rf_rd_data = rd_latch;

wire [7:0] lowmem_value, highmem_value, mem_value;
wire [5:0] highmem_rd_addr, highmem_wr_addr;

assign highmem_rd_addr = { rf_rd_bnk, rf_rd_addr[3:0] };
assign highmem_wr_addr = { rf_wr_bnk, rf_wr_addr[3:0] };
assign rd_wr_addr_equal = (rf_wr_addr == rf_rd_addr) &&
                          (!rf_wr_addr[4] || (rf_wr_bnk == rf_rd_bnk));

assign lowmem_value = lowmem[rf_rd_addr[3:0]];
assign highmem_value = highmem[highmem_rd_addr];
assign mem_value = rf_rd_addr[4] ? highmem_value : lowmem_value;

always @(posedge clk) begin
   rd_latch <= ((rd_wr_addr_equal && rf_we) ? rf_wr_data : mem_value);

   if(rf_we) begin
      $display("reg file write %h <== %h", { rf_wr_bnk, rf_wr_addr }, rf_wr_data);
      if(rf_wr_addr[4])
         highmem[highmem_wr_addr] <= rf_wr_data;
      else
         lowmem[rf_wr_addr[3:0]] <= rf_wr_data;
   end
end

endmodule
